# Copyright (c) 2021-2023 Alibaba Cloud Computing Ltd.
# SPDX-License-Identifier: MulanPSL-2.0
#!/usr/bin/python3
# -*- coding: utf-8 -*-
import paramiko
import re
import sys
import subprocess
import time
import logging

"""
Wrk Http long link benchmark
"""

logger = logging.getLogger(__name__)

# const
MYSQL_PASSWORD = ""
DATABASE_NAME = ""
TABLE_SIZE = 1000000
TABLE_NUMS = 44
REPORT_INTERVAL = 10
THREAD_NUMS = 40
TIMES = 120
PORT = 22
USER = 'root'
HOST_PASSWORD = ''


class Benchmark:
    def __init__(self, 
                url, 
                mysql_pwd = MYSQL_PASSWORD, 
                db_name = DATABASE_NAME, 
                tbl_size = TABLE_SIZE,
                tbl_nums = TABLE_NUMS, 
                rep_interval = REPORT_INTERVAL, 
                threads = THREAD_NUMS, 
                times = TIMES,
                port = PORT, 
                user = USER, 
                host_pwd = HOST_PASSWORD):
        """Init benchmark

        Args:
            url (string): url
            password (string): MySQL database password.
            db_name (int, optional):  table name.
            tbl_size (int, optional): The maximum size of table. Defaults to 10000.
            tbl_nums (int, optional): The maximum numbers of table. Defaults to 10.
            rep_interval (int, optional): The rep_interval of table. Defaults to 10.
            threads (int, optional): The numbers of thread. Defaults to 40.
            times (int, optional): The time. Defaults to 10.
        """
        self.url = str(url)
        self.mysql_pwd = mysql_pwd
        self.db_name = db_name
        self.tbl_size = tbl_size
        self.tbl_nums = tbl_nums
        self.rep_interval = rep_interval
        self.threads = threads
        self.time = times
        self.port = port
        self.user = user
        self.host_pwd = host_pwd

    def deploy_mysql(self):
        print("start deploy mysql")
        ssh = paramiko.SSHClient()
        key = paramiko.AutoAddPolicy()
        ssh.set_missing_host_key_policy(key)
        ssh.connect(self.url, self.port, self.user, self.host_pwd, timeout=60)
        _, stdout, stderr = ssh.exec_command('sh /home/my_sql.sh')
        if 'ok' in str(stdout.read().decode('utf-8')):
            status = True
            ssh.exec_command('mysql -u root -p{}'.format(self.mysql_pwd))
        else:
            status = False
        time.sleep(5)
        ssh.close()
        print("deploy mysql completely!")
        return status

    def prepare(self):
        cmd = '/usr/local/bin/sysbench /usr/local/share/sysbench/oltp_read_write.lua '\
                '--mysql-host={host} '\
                '--mysql-port=3306 ' \
                '--mysql-user=root '\
                '--mysql-password={password} '\
                '--db-driver=mysql '\
                '--mysql-db={db} ' \
                '--table-size={size} '\
                '--tables={tables} '\
                '--report-interval={interval} '\
                '--threads={threads} '\
                '--time={time} '\
                'cleanup'. \
            format(
                host = self.url, 
                password = self.mysql_pwd, 
                db = self.db_name, 
                size = self.tbl_size, 
                tables = self.tbl_nums,
                interval = self.rep_interval, 
                threads = self.threads, 
                time = self.time
            )
        
        print(cmd)
        result = subprocess.run(cmd, shell=True, close_fds=True, stderr=subprocess.PIPE, stdout=subprocess.PIPE)
        if result.returncode == 0:
            cmd = '/usr/local/bin/sysbench /usr/local/share/sysbench/oltp_read_write.lua ' \
                    '--mysql-host={host} '\
                    '--mysql-port=3306 ' \
                    '--mysql-user=root '\
                    '--mysql-password={password} '\
                    '--db-driver=mysql '\
                    '--mysql-db={db} ' \
                    '--table-size={size} ' \
                    '--tables={table_nums} ' \
                    '--report-interval={interval} ' \
                    '--threads={threads} ' \
                    '--time={time} prepare'. \
                format(
                    host = self.url, 
                    password = self.mysql_pwd, 
                    db = self.db_name, 
                    size = self.tbl_size, 
                    table_nums = self.tbl_nums,
                    interval = self.rep_interval, 
                    threads = self.threads, 
                    time = self.time
                )
            
            print(cmd)
            result = subprocess.run(cmd, shell=True, close_fds=True, stderr=subprocess.PIPE, stdout=subprocess.PIPE)
            if result.returncode == 0:
                logger.info("prepare database successfully")
                return True
            else:
                logger.error("prepare database failed")
                return False

        else:
            logger.error("cleanup database failed")
            return False

    def run(self):
        """Run benchmark and parse output

        Return True and score list if running benchmark successfully, otherwise return False and empty list.
        """
        if self.deploy_mysql():
            status = self.prepare()
        else:
            logger.error("deploy mysql failed")
            return False, []
        if status:
            cmd = '/usr/local/bin/sysbench /usr/local/share/sysbench/oltp_read_write.lua ' \
                    '--mysql-host={host} '\
                    '--mysql-port=3306 ' \
                    '--mysql-user=root '\
                    '--mysql-password={password} '\
                    '--db-driver=mysql '\
                    '--mysql-db={db} ' \
                    '--table-size={size} ' \
                    '--tables={table_nums} ' \
                    '--report-interval={interval} ' \
                    '--threads={threads} ' \
                    '--time={time} run'. \
                format(
                    host = self.url, 
                    password = self.mysql_pwd, 
                    db = self.db_name, 
                    size = self.tbl_size, 
                    table_nums = self.tbl_nums,
                    interval = self.rep_interval, 
                    threads = self.threads, 
                    time = self.time
                )
            
            print(cmd)
            result = subprocess.run(
                cmd,
                shell=True,
                close_fds=True,
                stderr=subprocess.PIPE,
                stdout=subprocess.PIPE
            )
            self.out = result.stdout.decode('UTF-8', 'strict')
            self.error = result.stderr.decode('UTF-8', 'strict')
            if result.returncode == 0:
                logger.info(self.out)

                pattern_static_trans = re.compile(r'transactions:\s+[\d.]+\s+\(([\d.]+)')
                pattern_static_queries = re.compile(r'queries:\s+[\d.]+\s+\(([\d.]+)')
                pattern_through_eps = re.compile(r'events/s \(eps\):\s+([\d.]+)')
                pattern_latency_avg = re.compile(r'avg:\s+([\d.]+)')
                pattern_latency_pct = re.compile(r'95th percentile:\s+([\d.]+)')

                if not re.search(pattern_static_trans, self.out) \
                        or not re.search(pattern_static_queries, self.out) \
                        or not re.search(pattern_through_eps, self.out) \
                        or not re.search(pattern_latency_avg, self.out) \
                        or not re.search(pattern_latency_pct, self.out):
                    logger.error("can not parse output: {}".format(self.out))
                    return False, []

                transactions = float(re.search(pattern_static_trans, self.out).group(1))
                queries = float(re.search(pattern_static_queries, self.out).group(1))
                events = float(re.search(pattern_through_eps, self.out).group(1))
                avg = float(re.search(pattern_latency_avg, self.out).group(1))
                percentile = float(re.search(pattern_latency_pct, self.out).group(1))

                result = {
                    "TPS": transactions,
                    "QPS": queries,
                    "EPS": events,
                    "Latency_avg": avg,
                    "Latency_pct": percentile
                }

                result_str = ", ".join(["{} = {}".format(k, v) for k, v in result.items()])
                print(result_str)
                return True, result_str

            else:
                logger.error(self.error)
                return False, []
        else:
            logger.error("prepare database failed")
            return False, []


if __name__ == "__main__":
    if sys.argv.__len__() <= 1:
        print("'Target ip' is wanted: python3 lto_mysql_http_read_write.py [Target ip]")
        exit(1)
    bench = Benchmark(sys.argv[1])
    suc, res = bench.run()
    print("The status is:", suc)
    print("The result is:", res)

